@@ -6,7 +6,7 @@ import matplotlib.pyplot as plt
 import matplotlib.gridspec as gridspec
 from scipy.optimize import brentq
 from .emissivity import *
-from astropy.cosmology import Planck15 as cosmo
+from astropy.cosmology import FlatLambdaCDM
 
 def plot_multi_profiles(profs, labels=None, outfile=None, axes=None, figsize=(13, 10), fontsize=40, xscale='log', yscale='log', fmt='o', markersize=7):
     """
@@ -108,7 +108,7 @@ class Profile(object):
     :type bins: class:`numpy.ndarray`
     """
     def __init__(self, data=None, center_choice=None, maxrad=None, binsize=None, center_ra=None, center_dec=None,
-                 binning='linear', centroid_region=None, bins=None):
+                 binning='linear', centroid_region=None, bins=None, cosmo=None):
         """
         Constructor of class Profile
         """
@@ -224,7 +224,13 @@ class Profile(object):
 
             else:
 
-                gsb = gaussian_filter(data.img, smc)
+                timg = np.copy(data.img)
+
+                zeroexp = np.where(data.exposure<=0.0)
+
+                timg[zeroexp] = 0.0
+
+                gsb = gaussian_filter(timg, smc)
 
             maxval = np.max(gsb)
 
@@ -292,6 +298,14 @@ class Profile(object):
         self.ccf = None
         self.lumfact = None
         self.box = False
+        self.anglow = 0.
+        self.anghigh = 360.
+        self.binning = binning
+        if data.voronoi:
+            self.voronoi = True
+        else:
+            self.voronoi = False
+
         if binning=='log':
             self.islogbin = True
         elif binning=='linear':
@@ -305,7 +319,13 @@ class Profile(object):
             print('Unknown binning option '+binning+', reverting to linear')
             self.islogbin = False
 
-    def SBprofile(self, ellipse_ratio=1.0, rotation_angle=0.0, angle_low=0., angle_high=360., voronoi=False, box=False, width=None):
+        if cosmo is None:
+            from astropy.cosmology import Planck15 as cosmo
+        self.cosmo = cosmo
+        self.scatter = None
+        self.escat = None
+
+    def SBprofile(self, ellipse_ratio=1.0, rotation_angle=0.0, angle_low=0., angle_high=360., box=False, width=None):
         """
         Extract a surface brightness profile and store the results in the input Profile object
 
@@ -326,7 +346,11 @@ class Profile(object):
         """
         data = self.data
         img = data.img
-        exposure = data.exposure
+        voronoi = self.voronoi
+        if not voronoi:
+            exposure = data.exposure
+        else:
+            exposure = data.errmap
         bkg = data.bkg
         pixsize = data.pixsize
         if not self.custom:
@@ -361,6 +385,8 @@ class Profile(object):
         xtil = np.cos(ellang) * (x - self.cx) * pixsize + np.sin(ellang) * (y - self.cy) * pixsize
         ytil = -np.sin(ellang) * (x - self.cx) * pixsize + np.cos(ellang) * (y - self.cy) * pixsize
         rads = ellipse_ratio * np.hypot(xtil, ytil / ellipse_ratio)
+        self.anglow = angle_low
+        self.anghigh = angle_high
         # Convert degree to radian and rescale to 0-2pi
         if angle_low != 0.0 and angle_high != 360.:
             if angle_low < 0.0:
@@ -419,8 +445,8 @@ class Profile(object):
             nv = len(img[id])
             if voronoi:
                 errmap = data.errmap
-                profile[i] = np.average(img[id], weights=1. / errmap[id] ** 2)
-                eprof[i] = np.sqrt(1. / np.sum(1. / errmap[id] ** 2))
+                profile[i] = np.sum(img[id]) / nv
+                eprof[i] = np.sqrt(np.sum(errmap[id] ** 2)) / nv
                 area[i] = nv * pixsize ** 2
                 effexp[i] = 1. # Dummy, but to be consistent with PSF calculation
             else:
@@ -451,7 +477,7 @@ class Profile(object):
             self.bkgcounts = bkgcounts
 
 
-    def MedianSB(self):
+    def MedianSB(self, ellipse_ratio=1.0, rotation_angle=0.0, outsamples=None):
         """
         Extract the median surface brightness profile in circular annuli from a provided Voronoi binned image, following the method outlined in Eckert et al. 2015
 
@@ -459,33 +485,151 @@ class Profile(object):
         data = self.data
         img = data.img
         errmap = data.errmap
+        expo = data.exposure
         if errmap is None:
             print('Error: No Voronoi map has been loaded')
             return
         pixsize = data.pixsize
-        if (self.islogbin):
-            self.bins, self.ebins = logbinning(self.binsize, self.maxrad)
-            nbin = len(self.bins)
-            self.nbin = nbin
-        else:
-            nbin = int(self.maxrad / self.binsize * 60. + 0.5)
-            self.bins = np.arange(self.binsize / 60. / 2., (nbin + 0.5) * self.binsize / 60., self.binsize / 60.)
-            self.ebins = np.ones(nbin) * self.binsize / 60. / 2.
-            self.nbin = nbin
-        profile, eprof, area, effexp = np.empty(self.nbin), np.empty(self.nbin), np.empty(self.nbin), np.empty(self.nbin)
+        if not self.custom:
+            if (self.islogbin):
+                self.bins, self.ebins = logbinning(self.binsize, self.maxrad)
+                nbin = len(self.bins)
+                self.nbin = nbin
+            else:
+                nbin = int(self.maxrad / self.binsize * 60. + 0.5)
+                self.bins = np.arange(self.binsize / 60. / 2., (nbin + 0.5) * self.binsize / 60., self.binsize / 60.)
+                self.ebins = np.ones(nbin) * self.binsize / 60. / 2.
+                self.nbin = nbin
+        #profile, eprof, area, effexp = np.empty(self.nbin), np.empty(self.nbin), np.empty(self.nbin), np.empty(self.nbin)
         y, x = np.indices(data.axes)
-        rads = np.sqrt((x - self.cx) ** 2 + (y - self.cy) ** 2) * pixsize
-        for i in range(self.nbin):
-            id = np.where(np.logical_and(
-                np.logical_and(rads >= self.bins[i] - self.ebins[i], rads < self.bins[i] + self.ebins[i]),
-                errmap > 0.0))  # left-inclusive
-            profile[i], eprof[i] = medianval(img[id], errmap[id], 1000)
-            area[i] = len(img[id]) * pixsize ** 2
-            effexp[i] = 1. # Dummy, but to be consistent with PSF calculation
+        if rotation_angle is not None:
+            self.ellangle = rotation_angle
+        else:
+            self.ellangle = 0.0
+
+        if ellipse_ratio is not None:
+            self.ellratio = ellipse_ratio
+        else:
+            self.ellratio = 1.0
+        tta = rotation_angle - 90.
+        if tta < -90. or tta > 270.:
+            print('Error: input angle must be between 0 and 360 degrees')
+            return
+        ellang = tta * np.pi / 180.
+        xtil = np.cos(ellang) * (x - self.cx) * pixsize + np.sin(ellang) * (y - self.cy) * pixsize
+        ytil = -np.sin(ellang) * (x - self.cx) * pixsize + np.cos(ellang) * (y - self.cy) * pixsize
+        rads = ellipse_ratio * np.hypot(xtil, ytil / ellipse_ratio)
+        #for i in range(self.nbin):
+        #    id = np.where(np.logical_and(
+        #        np.logical_and(np.logical_and(rads >= self.bins[i] - self.ebins[i], rads < self.bins[i] + self.ebins[i]),
+        #        errmap > 0.0),expo>0.0))  # left-inclusive
+        #    profile[i], eprof[i] = medianval(img[id], errmap[id], 1000)
+        #    area[i] = len(img[id]) * pixsize ** 2
+        #    effexp[i] = 1. # Dummy, but to be consistent with PSF calculation
+
+        all_prof, area = median_all_cov(data, self.bins, self.ebins, rads, nsim=1000)
+        profile, eprof = np.median(all_prof, axis=1), np.std(all_prof, axis=1)
+        effexp = np.ones(self.nbin) # Dummy, but to be consistent with PSF calculation
+        cov = np.cov(all_prof)
         self.profile = profile
         self.eprof = eprof
         self.area = area
         self.effexp = effexp
+        self.cov = cov
+
+        if outsamples is not None:
+            hdu = fits.PrimaryHDU(all_prof)
+            hdu.writeto(outsamples)
+
+    def AzimuthalScatter(self, nsect=12, model=None):
+
+        if self.profile is None:
+            print('Error: please extract a SB profile first')
+            return
+        dat = self.data
+        exposure = dat.exposure
+        img = dat.img
+
+        y, x = np.indices(dat.axes)
+        # Compute angles and set them between 0 and 2pi
+        angles = np.arctan2(y - self.cy, x - self.cx)
+        aneg = np.where(angles < 0.)
+        angles[aneg] = angles[aneg] + 2. * np.pi
+
+        all_sb = np.empty((self.nbin, nsect))
+        all_err = np.empty((self.nbin, nsect))
+
+        ellang = (self.ellangle - 90.) * np.pi / 180.
+        xtil = np.cos(ellang) * (x - self.cx) * dat.pixsize + np.sin(ellang) * (y - self.cy) * dat.pixsize
+        ytil = -np.sin(ellang) * (x - self.cx) * dat.pixsize + np.cos(ellang) * (y - self.cy) * dat.pixsize
+        rads = self.ellratio * np.hypot(xtil, ytil / self.ellratio)
+
+        skybkg = 0.
+        skybkg_err = 0.
+        if model is not None:
+            tp = 0
+            for p in range(model.npar):
+                if model.parnames[p] == 'bkg':
+                    tp = p
+            skybkg = np.power(10., model.params[tp])
+            skybkg_err = skybkg * np.log(10.) * model.errors[tp]
+
+        tol = 1e-5
+        for i in range(self.nbin):
+            anglow = 0.
+            anghigh = 2. * np.pi / nsect
+            for ns in range(nsect):
+                if i == 0:
+                    id = np.where(
+                        np.logical_and(np.logical_and(np.logical_and(np.logical_and(rads >= 0, rads < np.round(self.bins[i] + self.ebins[i], 5) + tol),
+                                       exposure > 0.0), angles >= anglow), angles <= anghigh))
+                else:
+                    id = np.where(np.logical_and(np.logical_and(np.logical_and(np.logical_and(rads >= np.round(self.bins[i] - self.ebins[i], 5) + tol,
+                                                rads < np.round(self.bins[i] + self.ebins[i], 5) + tol), exposure > 0.0), angles >= anglow), angles <= anghigh))
+            #            id=np.where(np.logical_and(np.logical_and(rads>=self.bins[i]-self.ebins[i],rads<self.bins[i]+self.ebins[i]),exposure>0.0)) #left-inclusive
+                nv = len(img[id])
+                if dat.voronoi:
+                    errmap = dat.errmap
+                    all_sb[i, ns] = np.sum(img[id]) / nv
+                    all_err[i, ns] = np.sqrt(np.sum(errmap[id] ** 2)) / nv
+                else:
+                    if nv > 0:
+                        bkgprof = np.sum(dat.bkg[id] / exposure[id]) / nv / dat.pixsize ** 2
+                        all_sb[i, ns] = np.sum(img[id] / exposure[id]) / nv / dat.pixsize ** 2 - bkgprof - skybkg
+                        all_err[i, ns] = np.sqrt(np.sum(img[id] / exposure[id] ** 2)) / nv / dat.pixsize ** 2
+                        all_err[i, ns] = np.sqrt(all_err[i, ns]**2 + skybkg_err**2)
+                    else:
+                        all_sb[i, ns] = 0.
+                        all_err[i, ns] = 0.
+
+                anglow = anglow + 2. * np.pi / nsect
+                anghigh = anghigh + 2. * np.pi / nsect
+
+        prof_mul = np.repeat(self.profile, nsect).reshape(self.nbin, nsect)
+        statscat = 1./nsect * np.sum(all_err**2 / prof_mul**2, axis=1)
+
+        nsim = 100
+        emul = np.repeat(all_err, nsim).reshape(self.nbin, nsect, nsim)
+        all_sb_mul = np.repeat(all_sb, nsim).reshape(self.nbin, nsect, nsim)
+
+        realiz = all_sb_mul + emul * np.random.randn(self.nbin, nsect, nsim)
+
+        profsect = np.repeat(self.profile, nsect).reshape(self.nbin, nsect)
+
+        profmul = np.repeat(profsect, nsim).reshape(self.nbin, nsect, nsim)
+
+        totscat_mul = 1. / nsect * np.sum((realiz - profmul) ** 2 / profmul ** 2, axis=1)
+
+        statscat_mul = np.repeat(statscat, nsim).reshape(self.nbin, nsim)
+
+        allvars = totscat_mul - statscat_mul
+        negscat = np.where(allvars < 0.)
+        allvars[negscat] = 0.
+
+        self.scatter = np.median(np.sqrt(allvars), axis=1)
+
+        self.err_scat = np.std(np.sqrt(allvars), axis=1)
+
 
     def Save(self, outfile=None, model=None):
         """
@@ -507,12 +651,15 @@ class Profile(object):
                 cols.append(fits.Column(name='WIDTH', format='E', unit='arcmin', array=self.ebins))
                 cols.append(fits.Column(name='SB', format='E', unit='cts s-1 arcmin-2', array=self.profile))
                 cols.append(fits.Column(name='ERR_SB', format='E', unit='cts s-1 arcmin-2', array=self.eprof))
+                cols.append(fits.Column(name='AREA', format='E', unit='arcmin2', array=self.area))
+                cols.append(fits.Column(name='EFFEXP', format='E', unit='s', array=self.effexp))
                 if self.counts is not None:
-                    cols.append(fits.Column(name='COUNTS', format='I', unit='', array=self.counts))
-                    cols.append(fits.Column(name='AREA', format='E', unit='arcmin2', array=self.area))
-                    cols.append(fits.Column(name='EFFEXP', format='E', unit='s', array=self.effexp))
                     cols.append(fits.Column(name='BKG', format='E', unit='cts s-1 arcmin-2', array=self.bkgprof))
+                    cols.append(fits.Column(name='COUNTS', format='I', unit='', array=self.counts))
                     cols.append(fits.Column(name='BKGCOUNTS', format='E', unit='', array=self.bkgcounts))
+                if self.scatter is not None:
+                    cols.append(fits.Column(name='SCATTER', format='E', array=self.scatter))
+                    cols.append(fits.Column(name='ERR_SCATTER', format='E', array=self.err_scat))
                 cols = fits.ColDefs(cols)
                 tbhdu = fits.BinTableHDU.from_columns(cols, name='DATA')
                 hdr = tbhdu.header
@@ -524,12 +671,36 @@ class Profile(object):
                 hdr['DEC_C'] = self.cdec
                 hdr.comments['RA_C'] = 'Right ascension of center value'
                 hdr.comments['DEC_C'] = 'Declination of center value'
-                hdr['COMMENT'] = 'Written by pyproffit (Eckert et al. 2011)'
+                hdr['ROT_ANGLE'] = self.ellangle
+                hdr['ELL_RATIO'] = self.ellratio
+                hdr.comments['ROT_ANGLE'] = 'Ellipse rotation angle'
+                hdr.comments['ELL_RATIO'] = 'Ellipse major-to-minor-axis ratio'
+                hdr['ANGLOW'] = self.anglow
+                hdr['ANGHIGH'] = self.anghigh
+                hdr.comments['ANGLOW'] = 'Lower position angle for sector definition'
+                hdr.comments['ANGHIGH'] = 'Upper position angle for sector definition'
+                hdr['BINSIZE'] = self.binsize
+                hdr['MAXRAD'] = self.maxrad
+                hdr.comments['BINSIZE'] = 'Profile bin size in arcsec'
+                hdr.comments['MAXRAD'] = 'Profile maximum radius in arcmin'
+                hdr['BINNING'] = self.binning
+                hdr.comments['BINNING'] = 'Binning scheme (linear, log or custom)'
+                hdr['IMAGE'] = self.data.imglink
+                hdr.comments['IMAGE'] = 'Path to input image file'
+                hdr['EXPMAP'] = self.data.explink
+                hdr.comments['EXPMAP'] = 'Path to exposure map file'
+                hdr['BKGMAP'] = self.data.bkglink
+                hdr.comments['BKGMAP'] = 'Path to background map file'
+                hdr['RMSMAP'] = self.data.rmsmap
+                hdr.comments['RMSMAP'] = 'Path to RMS file'
+                hdr['VORONOI'] = self.data.voronoi
+                hdr.comments['VORONOI'] = 'Voronoi on/off switch'
+                hdr['COMMENT'] = 'Written by pyproffit (Eckert et al. 2020)'
                 hdul.append(tbhdu)
             if model is not None:
                 cols = []
                 npar = len(model.params)
-                plist = np.arange(1,npar+1,1)
+                plist = np.arange(1, npar + 1, 1)
                 cols.append(fits.Column(name='PAR', format='1I', array=plist))
                 cols.append(fits.Column(name='NAME', format='16A', array=model.parnames))
                 cols.append(fits.Column(name='VALUE', format='E', array=model.params))
@@ -641,11 +812,11 @@ class Profile(object):
                     psfout[n, p] = np.sum(blurred[sn])
             self.psfmat = psfout
 
-    def SaveModelImage(self, model, outfile, vignetting=True):
+    def SaveModelImage(self, outfile, model=None, vignetting=True):
         """
         Compute a model image and output it to a FITS file
 
-        :param model: Object of type :class:`pyproffit.models.Model` including the surface brightness model
+        :param model: Object of type :class:`pyproffit.models.Model` including the surface brightness model. If model=None (default), the azimuthally averaged radial profile is interpolated at each point.
         :type model: class:`pyproffit.models.Model`
         :param outfile: Name of output file
         :type outfile: str
@@ -665,11 +836,14 @@ class Profile(object):
         xtil = np.cos(ellang) * (x - self.cx) * pixsize + np.sin(ellang) * (y - self.cy) * pixsize
         ytil = -np.sin(ellang) * (x - self.cx) * pixsize + np.cos(ellang) * (y - self.cy) * pixsize
         rads = ellipse_ratio * np.hypot(xtil, ytil / ellipse_ratio)
-        outmod = lambda x: model.model(x, *model.params)
-        if vignetting:
-            modimg = outmod(rads) * pixsize ** 2 * self.data.exposure
+        if model is not None:
+            outmod = model.model(rads, *model.params)
         else:
-            modimg = outmod(rads) * pixsize ** 2
+            outmod = np.interp(rads, self.bins, self.profile)
+        if vignetting:
+            modimg = outmod * pixsize ** 2 * self.data.exposure
+        else:
+            modimg = outmod * pixsize ** 2
         smoothing_scale=25
         gsb = gaussian_filter(self.data.bkg, smoothing_scale)
         gsexp = gaussian_filter(self.data.exposure, smoothing_scale)
@@ -730,8 +904,11 @@ class Profile(object):
         ax.minorticks_on()
         ax.tick_params(length=20, width=1, which='major', direction='in', right=True, top=True)
         ax.tick_params(length=10, width=1, which='minor', direction='in', right=True, top=True)
-        for item in (ax.get_xticklabels() + ax.get_yticklabels()):
+        for item in (ax.get_yticklabels()):
             item.set_fontsize(18)
+        for item in (ax.get_xticklabels()):
+            item.set_fontsize(0)
+
         if model is None:
             plt.xlabel('Radius [arcmin]', fontsize=fontsize)
             plt.ylabel('SB [cts/s/arcmin$^2$]', fontsize=fontsize)
@@ -750,13 +927,7 @@ class Profile(object):
         if model is not None:
             tmod = model.model(rads, *model.params)
             if self.psfmat is not None:
-                rminus = rads - self.ebins
-
-                rplus = rads + self.ebins
-
-                area = np.pi * (rplus ** 2 - rminus ** 2)
-
-                tmod = np.dot(np.transpose(self.psfmat),tmod * area) / area
+                tmod = np.dot(self.psfmat, tmod)
 
             plt.plot(rads, tmod, color=model_color, lw=lw, label='Model')
         xmin = rads[0] * 0.9
@@ -853,7 +1024,7 @@ class Profile(object):
         :return: Conversion factor
         :rtype: float
         """
-        self.ccf, self.lumfact = calc_emissivity(cosmo=cosmo,
+        self.ccf, self.lumfact = calc_emissivity(cosmo=self.cosmo,
                                         z=z,
                                         nh=nh,
                                         kt=kt,
@@ -866,4 +1037,4 @@ class Profile(object):
                                         lum_elow=lum_elow,
                                         lum_ehigh=lum_ehigh)
 
-        return self.ccf
\ No newline at end of file
+        return self.ccf
